from getgauge.messages.messages_pb2 import Message, StepValidateResponse
from getgauge.registry import registry


def validate_step(request, response):
    response.messageType = Message.StepValidateResponse
    response.stepValidateResponse.isValid = True
    if registry.is_implemented(request.stepValidateRequest.stepText) is False:
        response.stepValidateResponse.errorType = StepValidateResponse.STEP_IMPLEMENTATION_NOT_FOUND
        response.stepValidateResponse.errorMessage = 'Step implementation not found'
        response.stepValidateResponse.isValid = False
    elif registry.has_multiple_impls(request.stepValidateRequest.stepText):
        response.stepValidateResponse.isValid = False
        response.stepValidateResponse.errorType = StepValidateResponse.DUPLICATE_STEP_IMPLEMENTATION
        response.stepValidateResponse.errorMessage = 'Multiple implementation found for `{}` ({})'. \
            format(request.stepValidateRequest.stepText, ', '.join(
            ['{}:{}'.format(impl.file_name, impl.line_number) for impl in
             registry.get_infos_for(request.stepValidateRequest.stepText)]))



import random
import re

from getgauge import connection
from getgauge.messages.api_pb2 import APIMessage

_api = None


def get_step_value(step_text):
    global _api
    if _api is None:
        return re.sub('<[^<]+?>', '{}', step_text)
    req = APIMessage()
    req.stepValueRequest.stepText = step_text
    req.stepValueRequest.hasInlineTable = False
    req.messageType = APIMessage.GetStepValueRequest
    req.messageId = random.getrandbits(8)
    connection.send_message(req, req, _api)
    res = connection.read_message(_api, APIMessage())
    return res.stepValueResponse.stepValue.stepValue


def connect_to_api():
    global _api
    _api = connection.to_api()


def close_api():
    global _api
    _api = _api.close()
